// ////////////////////////////////////////////////////
//
// StdView.cpp : implementation of the CStdView class
// Based on class COGLView : public CView
// Inspect COGLView for standard OpenGL initialization
//
////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Std.h"

#include "StdDoc.h"
#include "StdView.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////
// You can put definitions, variables and helperfunctions
// here. Functions that does not rely on CStdView member
// variables or functions
// definitions according to assignment
#define R	2.0f
#define H	8.0f

//////////////////////////////////////////////////////////
// CStdView

IMPLEMENT_DYNCREATE(CStdView, COGLView)

BEGIN_MESSAGE_MAP(CStdView, COGLView)
	//{{AFX_MSG_MAP(CStdView)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////
// CStdView construction/destruction

CStdView::CStdView()
{
	// initialize angles, see DrawScene
	m_xv=0.0f;
	m_yv=0.0f;
	m_hRC=NULL;
	m_hDC=NULL;
};

CStdView::~CStdView()
{}

////////////////////////////////////////////////////////////
// CStdView diagnostics
// You should usually not touch these:
#ifdef _DEBUG
void CStdView::AssertValid() const
{COGLView::AssertValid();}
void CStdView::Dump(CDumpContext& dc) const
{COGLView::Dump(dc);}

CStdDoc* CStdView::GetDocument() // non-debug version is inline
{	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CStdDoc)));
	return (CStdDoc*)m_pDocument;
}
#endif //_DEBUG

//////////////////////////////////////////////////////////////
// CStdView operations

void CStdView::DrawScene()
{
	// Clear the color and depth buffers.
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		
	// Set up for scene
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// put origo in houses soth-west corner
	gluLookAt(	4*R,-4*R,H/2.0f,
				0.0f,0.0f,H/2.0f,
				0.0f,0.0f,1.0f);

	////////////////////////////////////
	// pick up user manipulation
	// rotate around x-axis 
	glRotatef(m_xv,1.0f,0.0f,0.0f);
	// rotate around z-axis 
	glRotatef(m_yv,0.0f,0.0f,1.0f);
	/////////////////////////////////////

	// base cone
	CMaterial::SetMaterial(WARM_WHITE);
	GLUquadricObj *pQ;
	pQ = gluNewQuadric();
	gluCylinder(pQ,R,0.0f,H,50,50); 
	gluDeleteQuadric(pQ);

	// spiral with m circles
	CMaterial::SetMaterial(RED_PLASTIC);
	GLfloat m=15.0f;
	GLfloat t=0.0f;
	GLfloat x,y,z;
	GLfloat incR=R*1.05f;
	glBegin(GL_LINE_STRIP);
	while(t<m)
	{
		x=incR*(m-t)/m*(GLfloat)cos(6.28*t);
		y=incR*(m-t)/m*(GLfloat)sin(6.28*t);
		z=H/m*t;
		glVertex3f(x,y,z);
		t+=0.05f;
	}
	glEnd();

		
	// finnish drawing
	glFlush();
}

void CStdView::InitializeRContext()
{
	// Here we set the attributes that will be constant
	// during the rendering contexts lifetime
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LESS);
    
	// prepare ligthsource
	GLfloat ambient[] = {0.2f,0.2f,0.2f,1.0f };
    GLfloat diffuse[] = {1.0f,1.0f,1.0f,1.0f };
    GLfloat position[] = {4.0f,0.0f,6.0f,0.0f };
	GLfloat lmodel_ambient[] = {0.4f,0.4f,0.4f,1.0f };


    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
    glLightfv(GL_LIGHT0, GL_POSITION, position);
	
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);

	// smooth the drawing
	glShadeModel(GL_SMOOTH);

    // set background to light gray
	//glClearColor(0.8f, 0.8f, 0.8f, 0.0f);
	glClearColor(1.0f, 1.0f, 1.0f, 0.0f);
}

void CStdView::SetPerspective(int cx, int cy)
{
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	gluPerspective(50.0f, (GLdouble)cx/cy, 0.1f, 70.0f);
	glMatrixMode(GL_MODELVIEW);
}


void CStdView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// User has pressed left button.Make a note
	// of this and prepare for intuitive rotation of scene
	m_PLast=point;
	SetCapture();
	CView::OnLButtonDown(nFlags, point);
}

void CStdView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// free capture and terminate rotation
	ReleaseCapture();
	CView::OnLButtonUp(nFlags, point);
}

void CStdView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CRect cR;
	// if left button is down 
	if(nFlags & MK_LBUTTON)
	{	
		// and has moved a significant dist
		if( (abs(m_PLast.x-point.x)>2)||(abs(m_PLast.y-point.y)>2) )
		{
			// then we find an intuitive rot angle
			GetClientRect(&cR);
			
			m_yv+=90.0f*((GLfloat (point.x-m_PLast.x))/GLfloat (cR.Width()));
			m_xv+=90.0f*((GLfloat (point.y-m_PLast.y))/GLfloat (cR.Height()));
			
			// rememeber point and force immediate redraw 
			m_PLast=point;
			RedrawWindow();
		}
	}
}
